@@ -27,12 +27,12 @@ class ClassLinkedMap : public LinkedMap<ClassDef>
 {
 };
 
-class ClassLinkedRefMap : public LinkedRefMap<const ClassDef>
+class ClassLinkedRefMap : public LinkedRefMap<ClassDef>
 {
   public:
-    bool declVisible(const ClassDef::CompoundType *filter=0) const;
+    bool declVisible(const ClassDef::CompoundType *filter=nullptr) const;
     void writeDeclaration(OutputList &ol,const ClassDef::CompoundType *filter,const QCString &header,bool localNames) const;
-    void writeDocumentation(OutputList &ol,const Definition * container=0) const;
+    void writeDocumentation(OutputList &ol,const Definition * container=nullptr) const;
 };
 
 #endif
