@@ -32,6 +32,7 @@
 #include "section.h"
 #include "construct.h"
 #include "doctokenizer.h"
+#include "cite.h"
 
 class MemberDef;
 class Definition;
@@ -51,7 +52,7 @@ class DocParser;
 /* 40 */  DN(DocSimpleSect)     DN_SEP DN(DocSimpleSectSep) DN_SEP DN(DocParamSect)      DN_SEP DN(DocPara)         DN_SEP DN(DocParamList)   DN_SEP   \
 /* 45 */  DN(DocSimpleListItem) DN_SEP DN(DocHtmlListItem)  DN_SEP DN(DocHtmlDescData)   DN_SEP DN(DocHtmlCell)     DN_SEP DN(DocHtmlCaption) DN_SEP   \
 /* 50 */  DN(DocHtmlRow)        DN_SEP DN(DocHtmlTable)     DN_SEP DN(DocHtmlBlockQuote) DN_SEP DN(DocText)         DN_SEP DN(DocRoot)        DN_SEP   \
-/* 55 */  DN(DocHtmlDetails)    DN_SEP DN(DocHtmlSummary)                                                                                              \
+/* 55 */  DN(DocHtmlDetails)    DN_SEP DN(DocHtmlSummary)   DN_SEP DN(DocPlantUmlFile)                                                                      \
 
 // forward declarations
 #define DN(x) class x;
@@ -243,19 +244,22 @@ class DocAnchor : public DocNode
 class DocCite : public DocNode
 {
   public:
-    DocCite(DocParser *parser,DocNodeVariant *parent,const QCString &target,const QCString &context);
+    DocCite(DocParser *parser,DocNodeVariant *parent,const QCString &target,const QCString &context, CiteInfoOption opt);
     QCString file() const        { return m_file; }
     QCString relPath() const     { return m_relPath; }
     QCString ref() const         { return m_ref; }
     QCString anchor() const      { return m_anchor; }
-    QCString text() const        { return m_text; }
+    QCString target() const      { return m_target; }
+    CiteInfoOption option() const  { return m_option; }
+    QCString getText() const;
 
   private:
     QCString   m_file;
     QCString   m_relPath;
     QCString   m_ref;
     QCString   m_anchor;
-    QCString   m_text;
+    QCString   m_target;
+    CiteInfoOption m_option;
 };
 
 
@@ -278,7 +282,9 @@ class DocStyleChange : public DocNode
                  Del           = (1<<12),
                  Ins           = (1<<13),
                  S             = (1<<14),
-                 Cite          = (1<<15)
+                 Cite          = (1<<15),
+                 Kbd           = (1<<16),
+                 Typewriter    = (1<<17)
                };
 
     DocStyleChange(DocParser *parser,DocNodeVariant *parent,size_t position,Style s,
@@ -288,12 +294,24 @@ class DocStyleChange : public DocNode
       if (attribs) m_attribs=*attribs;
       m_tagName = tagName.lower();
     }
+    DocStyleChange(DocParser *parser,DocNodeVariant *parent,size_t position,Style s,
+                   const QCString &tagName,bool enable,
+                   const QCString &fileName,int lineNr,
+                   const HtmlAttribList *attribs=nullptr)
+      : DocNode(parser,parent), m_position(position), m_style(s), m_enable(enable),
+        m_fileName(fileName), m_lineNr(lineNr)
+    {
+      if (attribs) m_attribs=*attribs;
+      m_tagName = tagName.lower();
+    }
     Style style() const                   { return m_style; }
     const char *styleString() const;
     bool enable() const                   { return m_enable; }
     size_t position() const               { return m_position; }
     const HtmlAttribList &attribs() const { return m_attribs; }
     QCString tagName() const              { return m_tagName; }
+    QCString fileName() const             { return m_fileName; }
+    int lineNr() const                    { return m_lineNr; }
 
   private:
     size_t   m_position = 0;
@@ -301,6 +319,8 @@ class DocStyleChange : public DocNode
     bool     m_enable = false;
     HtmlAttribList m_attribs;
     QCString m_tagName;
+    QCString m_fileName;
+    int      m_lineNr = -1;
 };
 
 /** Node representing a special symbol */
@@ -416,14 +436,15 @@ class DocInclude : public DocNode
   public:
   enum Type { Include, DontInclude, VerbInclude, HtmlInclude, LatexInclude,
 	      IncWithLines, Snippet , SnippetWithLines,
-	      DontIncWithLines, RtfInclude, ManInclude, DocbookInclude, XmlInclude,
-              SnippetTrimLeft};
+	      DontIncWithLines, RtfInclude, ManInclude, DocbookInclude, XmlInclude
+            };
     DocInclude(DocParser *parser,DocNodeVariant *parent,const QCString &file,
-               const QCString &context, Type t,
+               const QCString &context, Type t, bool stripCodeComments,
                bool isExample,const QCString &exampleFile,
-               const QCString &blockId, bool isBlock)
+               const QCString &blockId, bool isBlock, bool trimLeft)
     : DocNode(parser,parent), m_file(file), m_context(context), m_type(t),
-      m_isExample(isExample), m_isBlock(isBlock),
+      m_stripCodeComments(stripCodeComments),
+      m_isExample(isExample), m_isBlock(isBlock), m_trimLeft(trimLeft),
       m_exampleFile(exampleFile), m_blockId(blockId) {}
     QCString file() const        { return m_file; }
     QCString extension() const   { int i=m_file.findRev('.'); return i!=-1 ? m_file.mid(i) : QCString(); }
@@ -431,9 +452,11 @@ class DocInclude : public DocNode
     QCString text() const        { return m_text; }
     QCString context() const     { return m_context; }
     QCString blockId() const     { return m_blockId; }
+    bool stripCodeComments() const { return m_stripCodeComments; }
     bool isExample() const       { return m_isExample; }
     QCString exampleFile() const { return m_exampleFile; }
     bool isBlock() const         { return m_isBlock; }
+    bool trimLeft() const        { return m_trimLeft; }
     void parse();
 
   private:
@@ -441,8 +464,10 @@ class DocInclude : public DocNode
     QCString  m_context;
     QCString  m_text;
     Type      m_type;
+    bool      m_stripCodeComments;
     bool      m_isExample;
     bool      m_isBlock;
+    bool      m_trimLeft;
     QCString  m_exampleFile;
     QCString  m_blockId;
 };
@@ -453,9 +478,9 @@ class DocIncOperator : public DocNode
   public:
     enum Type { Line, SkipLine, Skip, Until };
     DocIncOperator(DocParser *parser,DocNodeVariant *parent,Type t,const QCString &pat,
-                   const QCString &context,bool isExample,const QCString &exampleFile)
+                   const QCString &context,bool stripCodeComments,bool isExample,const QCString &exampleFile)
     : DocNode(parser,parent), m_type(t), m_pattern(pat), m_context(context),
-      m_isFirst(FALSE), m_isLast(FALSE),
+      m_isFirst(FALSE), m_isLast(FALSE), m_stripCodeComments(stripCodeComments),
       m_isExample(isExample), m_exampleFile(exampleFile) {}
     Type type() const           { return m_type; }
     const char *typeAsString() const
@@ -478,6 +503,7 @@ class DocIncOperator : public DocNode
     bool isLast() const          { return m_isLast; }
     void markFirst(bool v=TRUE)  { m_isFirst = v; }
     void markLast(bool v=TRUE)   { m_isLast = v; }
+    bool stripCodeComments() const { return m_stripCodeComments; }
     bool isExample() const       { return m_isExample; }
     QCString exampleFile() const { return m_exampleFile; }
     QCString includeFileName() const { return m_includeFileName; }
@@ -492,6 +518,7 @@ class DocIncOperator : public DocNode
     QCString  m_context;
     bool     m_isFirst = false;
     bool     m_isLast = false;
+    bool     m_stripCodeComments = true;
     bool     m_isExample = false;
     QCString  m_exampleFile;
     QCString m_includeFileName;
@@ -526,7 +553,7 @@ class DocIndexEntry : public DocNode
   public:
     DocIndexEntry(DocParser *parser,DocNodeVariant *parent,const Definition *scope,const MemberDef *md)
       : DocNode(parser,parent), m_scope(scope), m_member(md) {}
-    Tokens parse();
+    Token parse();
     const Definition *scope() const    { return m_scope;  }
     const MemberDef *member() const    { return m_member; }
     QCString entry() const        { return m_entry;  }
@@ -554,7 +581,7 @@ class DocAutoList : public DocCompoundNode
     int  indent() const        { return m_indent; }
     bool isCheckedList() const { return m_isCheckedList; }
     int depth() const          { return m_depth; }
-    Tokens parse();
+    Token parse();
 
   private:
     int      m_indent = 0;
@@ -569,7 +596,7 @@ class DocAutoListItem : public DocCompoundNode
   public:
     DocAutoListItem(DocParser *parser,DocNodeVariant *parent,int indent,int num);
     int itemNumber() const     { return m_itemNum; }
-    Tokens parse();
+    Token parse();
 
   private:
     int m_indent = 0;
@@ -584,6 +611,7 @@ class DocTitle : public DocCompoundNode
     void parse();
     void parseFromString(DocNodeVariant *,const QCString &title);
     bool hasTitle() const { return !children().empty(); }
+    bool isEmpty() const  { return !hasTitle(); }
 
   private:
 };
@@ -707,6 +735,15 @@ class DocDiaFile : public DocDiagramFileBase
     bool parse();
 };
 
+/** Node representing a uml file */
+class DocPlantUmlFile : public DocDiagramFileBase
+{
+  public:
+    DocPlantUmlFile(DocParser *parser,DocNodeVariant *parent,const QCString &name,const QCString &context,
+               const QCString &srcFile,int srcLine);
+    bool parse();
+};
+
 /** Node representing a VHDL flow chart */
 class DocVhdlFlow : public DocCompoundNode
 {
@@ -789,7 +826,7 @@ class DocHRef : public DocCompoundNode
            const QCString &relPath, const QCString &file)
     : DocCompoundNode(parser,parent), m_attribs(attribs), m_url(url),
       m_relPath(relPath), m_file(file) {}
-    Tokens parse();
+    Token parse();
     QCString url() const        { return m_url; }
     QCString file() const       { return m_file; }
     QCString relPath() const    { return m_relPath; }
@@ -822,7 +859,7 @@ class DocHtmlDetails : public DocCompoundNode
     DocHtmlDetails(DocParser *parser,DocNodeVariant *parent,const HtmlAttribList &attribs) :
        DocCompoundNode(parser,parent), m_attribs(attribs) {}
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
+    Token parse();
     void parseSummary(DocNodeVariant *,HtmlAttribList &attribs);
     const DocNodeVariant *summary() const { return m_summary.get(); }
 
@@ -839,7 +876,7 @@ class DocHtmlHeader : public DocCompoundNode
        DocCompoundNode(parser,parent), m_level(level), m_attribs(attribs) {}
     int level() const                     { return m_level; }
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
+    Token parse();
 
   private:
     int           m_level = 0;
@@ -853,7 +890,7 @@ class DocHtmlDescTitle : public DocCompoundNode
     DocHtmlDescTitle(DocParser *parser,DocNodeVariant *parent,const HtmlAttribList &attribs) :
       DocCompoundNode(parser,parent), m_attribs(attribs) {}
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
+    Token parse();
 
   private:
     HtmlAttribList m_attribs;
@@ -866,7 +903,7 @@ class DocHtmlDescList : public DocCompoundNode
     DocHtmlDescList(DocParser *parser,DocNodeVariant *parent,const HtmlAttribList &attribs) :
       DocCompoundNode(parser,parent), m_attribs(attribs) {}
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
+    Token parse();
 
   private:
     HtmlAttribList m_attribs;
@@ -883,7 +920,7 @@ class DocSection : public DocCompoundNode
     QCString anchor() const     { return m_anchor; }
     QCString id() const         { return m_id; }
     QCString file() const       { return m_file; }
-    Tokens parse();
+    Token parse();
 
   private:
     int       m_level = 0;
@@ -932,7 +969,7 @@ class DocInternal : public DocCompoundNode
 {
   public:
     DocInternal(DocParser *parser,DocNodeVariant *parent) : DocCompoundNode(parser,parent) {}
-    Tokens parse(int);
+    Token parse(int);
 
   private:
 };
@@ -942,7 +979,7 @@ class DocParBlock : public DocCompoundNode
 {
   public:
     DocParBlock(DocParser *parser,DocNodeVariant *parent) : DocCompoundNode(parser,parent) {}
-    Tokens parse();
+    Token parse();
 
   private:
 };
@@ -953,7 +990,7 @@ class DocSimpleList : public DocCompoundNode
 {
   public:
     DocSimpleList(DocParser *parser,DocNodeVariant *parent) : DocCompoundNode(parser,parent) {}
-    Tokens parse();
+    Token parse();
 
   private:
 };
@@ -967,8 +1004,8 @@ class DocHtmlList : public DocCompoundNode
       DocCompoundNode(parser,parent), m_type(t), m_attribs(attribs) {}
     Type type() const          { return m_type; }
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
-    Tokens parseXml();
+    Token parse();
+    Token parseXml();
 
   private:
     Type           m_type = Unordered;
@@ -988,9 +1025,9 @@ class DocSimpleSect : public DocCompoundNode
     DocSimpleSect(DocParser *parser,DocNodeVariant *parent,Type t);
     Type type() const       { return m_type; }
     QCString typeString() const;
-    Tokens parse(bool userTitle,bool needsSeparator);
-    Tokens parseRcs();
-    Tokens parseXml();
+    Token parse(bool userTitle,bool needsSeparator);
+    Token parseRcs();
+    Token parseXml();
     void appendLinkWord(const QCString &word);
     bool hasTitle() const;
     const DocNodeVariant *title() const { return m_title.get(); }
@@ -1027,7 +1064,7 @@ class DocParamSect : public DocCompoundNode
     DocParamSect(DocParser *parser,DocNodeVariant *parent,Type t)
       : DocCompoundNode(parser,parent), m_type(t), m_hasInOutSpecifier(FALSE), m_hasTypeSpecifier(FALSE)
     {}
-    Tokens parse(const QCString &cmdName,bool xmlContext,Direction d);
+    Token parse(const QCString &cmdName,bool xmlContext,Direction d);
     Type type() const          { return m_type; }
     bool hasInOutSpecifier() const { return m_hasInOutSpecifier; }
     bool hasTypeSpecifier() const  { return m_hasTypeSpecifier; }
@@ -1043,19 +1080,19 @@ class DocPara : public DocCompoundNode
 {
   public:
     DocPara(DocParser *parser,DocNodeVariant *parent);
-    Tokens parse();
+    Token parse();
     bool isEmpty() const        { return children().empty(); }
     void markFirst(bool v=TRUE) { m_isFirst=v; }
     void markLast(bool v=TRUE)  { m_isLast=v; }
     bool isFirst() const        { return m_isFirst; }
     bool isLast() const         { return m_isLast; }
 
-    Tokens handleCommand(char cmdChar,const QCString &cmdName);
-    Tokens handleHtmlStartTag(const QCString &tagName,const HtmlAttribList &tagHtmlAttribs);
-    Tokens handleHtmlEndTag(const QCString &tagName);
-    Tokens handleSimpleSection(DocSimpleSect::Type t,bool xmlContext=FALSE);
-    Tokens handleXRefItem();
-    Tokens handleParamSection(const QCString &cmdName,DocParamSect::Type t, bool xmlContext, int direction);
+    Token handleCommand(char cmdChar,const QCString &cmdName);
+    Token handleHtmlStartTag(const QCString &tagName,const HtmlAttribList &tagHtmlAttribs);
+    Token handleHtmlEndTag(const QCString &tagName);
+    Token handleSimpleSection(DocSimpleSect::Type t,bool xmlContext=FALSE);
+    Token handleXRefItem();
+    Token handleParamSection(const QCString &cmdName,DocParamSect::Type t, bool xmlContext, int direction);
     void handleIncludeOperator(const QCString &cmdName,DocIncOperator::Type t);
     template<class T> void handleFile(const QCString &cmdName);
     void handleInclude(const QCString &cmdName,DocInclude::Type t);
@@ -1070,10 +1107,10 @@ class DocPara : public DocCompoundNode
     void handleILine(char cmdChar,const QCString &cmdName);
     void handleIFile(char cmdChar,const QCString &cmdName);
     void handleShowDate(char cmdChar,const QCString &cmdName);
-    Tokens handleStartCode();
-    Tokens handleHtmlHeader(const HtmlAttribList &tagHtmlAttribs,int level);
+    Token handleStartCode();
+    Token handleHtmlHeader(const HtmlAttribList &tagHtmlAttribs,int level);
 
-    bool injectToken(Tokens tok,const QCString &tokText);
+    bool injectToken(Token tok,const QCString &tokText);
     const HtmlAttribList &attribs() const { return m_attribs; }
     void setAttribs(const HtmlAttribList &attribs) { m_attribs = attribs; }
 
@@ -1098,8 +1135,8 @@ class DocParamList : public DocNode
     void markLast(bool b=TRUE)      { m_isLast=b; }
     bool isFirst() const            { return m_isFirst; }
     bool isLast() const             { return m_isLast; }
-    Tokens parse(const QCString &cmdName);
-    Tokens parseXml(const QCString &paramName);
+    Token parse(const QCString &cmdName);
+    Token parseXml(const QCString &paramName);
 
   private:
     DocNodeList             m_paragraphs;
@@ -1116,7 +1153,7 @@ class DocSimpleListItem : public DocNode
 {
   public:
     DocSimpleListItem(DocParser *parser,DocNodeVariant *parent);
-    Tokens parse();
+    Token parse();
     const DocNodeVariant *paragraph() const { return m_paragraph.get(); }
 
   private:
@@ -1131,8 +1168,8 @@ class DocHtmlListItem : public DocCompoundNode
     : DocCompoundNode(parser,parent), m_attribs(attribs), m_itemNum(num) {}
     int itemNumber() const                { return m_itemNum; }
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
-    Tokens parseXml();
+    Token parse();
+    Token parseXml();
 
   private:
     HtmlAttribList m_attribs;
@@ -1145,7 +1182,7 @@ class DocHtmlDescData : public DocCompoundNode
   public:
     DocHtmlDescData(DocParser *parser,DocNodeVariant *parent) : DocCompoundNode(parser,parent) {}
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
+    Token parse();
 
   private:
     HtmlAttribList m_attribs;
@@ -1166,8 +1203,8 @@ class DocHtmlCell : public DocCompoundNode
     void markFirst(bool v=TRUE) { m_isFirst=v; }
     void markLast(bool v=TRUE)  { m_isLast=v; }
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
-    Tokens parseXml();
+    Token parse();
+    Token parseXml();
     uint32_t rowIndex() const        { return m_rowIdx; }
     uint32_t columnIndex() const     { return m_colIdx; }
     uint32_t rowSpan() const;
@@ -1192,7 +1229,7 @@ class DocHtmlCaption : public DocCompoundNode
   public:
     DocHtmlCaption(DocParser *parser,DocNodeVariant *parent,const HtmlAttribList &attribs);
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
+    Token parse();
     bool hasCaptionId() const { return m_hasCaptionId; }
     QCString file() const     { return m_file;         }
     QCString anchor() const   { return m_anchor;       }
@@ -1213,8 +1250,8 @@ class DocHtmlRow : public DocCompoundNode
       : DocCompoundNode(parser,parent), m_attribs(attribs) {}
     size_t numCells() const      { return children().size(); }
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
-    Tokens parseXml(bool header);
+    Token parse();
+    Token parseXml(bool header);
     bool isHeading() const;
     void setVisibleCells(uint32_t n) { m_visibleCells = n; }
     uint32_t visibleCells() const    { return m_visibleCells; }
@@ -1236,8 +1273,8 @@ class DocHtmlTable : public DocCompoundNode
     size_t numRows() const  { return children().size(); }
     bool hasCaption() const;
     const HtmlAttribList &attribs() const { return m_attribs; }
-    Tokens parse();
-    Tokens parseXml();
+    Token parse();
+    Token parseXml();
     size_t numColumns() const { return m_numCols; }
     const DocNodeVariant *caption() const;
     const DocNodeVariant *firstRow() const;
@@ -1255,7 +1292,7 @@ class DocHtmlBlockQuote : public DocCompoundNode
   public:
     DocHtmlBlockQuote(DocParser *parser,DocNodeVariant *parent,const HtmlAttribList &attribs)
       : DocCompoundNode(parser,parent), m_attribs(attribs) {}
-    Tokens parse();
+    Token parse();
     const HtmlAttribList &attribs() const { return m_attribs; }
 
   private:
@@ -1445,6 +1482,10 @@ class DocNodeAST : public IDocNodeAST
       {
         return std::get<DocText>(root).isEmpty();
       }
+      else if (std::holds_alternative<DocTitle>(root))
+      {
+        return std::get<DocTitle>(root).isEmpty();
+      }
       return false;
     }
     DocNodeVariant root;
